#!/usr/bin/python


from libsan.host import fio, scsi
from libsan.host.iscsi import clean_up, discovery_st, node_login

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def do_io(device):  # fio - running randwrite until the 1G device is full, then verifying written data using CRC32C.
    if not fio.install_fio():
        print("FAIL: Unable to install fio")
        return False

    fio_params = {
        "bs": "4k",
        "direct": 1,
        "ioengine": "libaio",
        "iodepth": 16,
        "verify": "crc32c",
        "verify_fatal": 1,
        "do_verify": 1,
        "rw": "randwrite",
        "runtime": 120,
    }  # adding runtime cap of 2 minutes

    if not fio.fio_stress(device, verbose=True, **fio_params):
        print("FIO I/O failed")
        return False

    return True


def iscsi_local():
    errors = []
    ip_address = "127.0.0.1"
    target_iqn = read_var("TARGET_IQN")

    atomic_run(
        "Discovering local target",
        target=ip_address,
        disc_db=True,
        command=discovery_st,
        errors=errors,
    )

    atomic_run(
        "Trying to login to portal: %s" % ip_address,
        target=target_iqn,
        command=node_login,
        errors=errors,
    )

    ret = atomic_run(
        "Getting free scsi disks",
        filter_only={"vendor": "LIO-ORG"},
        command=scsi.get_free_disks,
        errors=errors,
    )

    try:
        test_dev = "/dev/" + list(ret.keys())[0]
    except IndexError:
        msg = "FAIL: Could not find any free SCSI disk!"
        print(msg)
        errors.append(msg)
        return errors

    atomic_run(
        "Running fio stress on device %s" % test_dev,
        device=test_dev,
        command=do_io,
        errors=errors,
    )

    atomic_run("Cleaning up", command=clean_up, errors=errors)

    return errors


if __name__ == "__main__":
    errs = iscsi_local()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.dmpd import DMPD
from libsan.host.linux import run
from libsan.host.lvm import lv_convert, lv_create, lv_deactivate

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string, read_env
from stqe.host.lvm import get_dmpd_args
from stqe.host.persistent_vars import read_var, write_var


def mount_lv(vg_name, lv_name):
    """Creates dir, filesystem and mounts LV. The purpose of this is to increase metadata volume.
    :param vg_name: VG with LV in it
    :type vg_name: string
    :param lv_name: LV name to be mounted
    :type lv_name: string
    :return: retcode
    :rtype: int.
    """
    ret = run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    ret += run(f"mkfs.ext4 /dev/{vg_name}/{lv_name}", verbose=True)
    ret += run(f"mount /dev/{vg_name}/{lv_name} /mnt/{lv_name}/", verbose=True)
    return ret


def umount_lv(lv_name):
    """Umounts LV and removes dir.
    :param lv_name: mounted LV name
    :type lv_name: string
    :return: retcode
    :rtype: int.
    """
    ret = run("umount -f /mnt/%s/" % lv_name, verbose=True)
    ret += run("rm -rf /mnt/%s" % lv_name, verbose=True)
    return ret


def metadata_snapshot(vg_name, thinpool):
    """Creates metadata snapshot of thinpool
    :param vg_name: VG with thinpool in it
    :type vg_name: string
    :param thinpool: thinpool name
    :type thinpool: string
    :return: retcode
    :rtype: int.
    """
    ret = run(f"dmsetup suspend /dev/mapper/{vg_name}-{thinpool}-tpool", verbose=True)
    ret += run(
        f"dmsetup message /dev/mapper/{vg_name}-{thinpool}-tpool 0 reserve_metadata_snap",
        verbose=True,
    )
    ret += run(f"dmsetup resume /dev/mapper/{vg_name}-{thinpool}-tpool", verbose=True)
    return ret


def create_lvs(vg_name, thinpool, lv_name, options, number_of_vols, return_output=True):
    """Creates few LVs to fill in pool metadata. This increases transaction ID and allows to do thin_delta
    :param vg_name: VG with thinpool in it
    :type vg_name: string
    :param thinpool: thinpool name
    :type thinpool: string
    :param lv_name: base of LVs name
    :type lv_name: string
    :param options: LV options to be used for creating LVs
    :type options: list
    :param number_of_vols: Number of LVs to create
    :type number_of_vols: int
    :param return_output: should just errors or also retcode? (default True - both)
    :type return_output: bool
    :return: retcode, list of errors
    :rtype: tuple.
    """
    errors = []
    for i in range(number_of_vols):
        atomic_run(
            "Creating thin LV No. %s" % i,
            vg_name=vg_name + "/" + thinpool,
            lv_name=lv_name + str(i),
            options=options,
            command=lv_create,
            errors=errors,
        )

        atomic_run(
            "Mounting LV No. %s" % i,
            expected_ret=0,
            vg_name=vg_name,
            lv_name=lv_name + str(i),
            command=mount_lv,
            errors=errors,
        )

        atomic_run(
            "Unmounting LV No. %s" % i,
            expected_ret=0,
            lv_name=lv_name + str(i),
            command=umount_lv,
            errors=errors,
        )

        atomic_run(
            "Deactivating thin LV No. %s" % i,
            vg_name=vg_name,
            lv_name=lv_name + str(i),
            command=lv_deactivate,
            errors=errors,
        )
    if return_output:
        if errors == []:
            return 0, errors
        return 1, errors
    else:
        return errors


def swap_metadata(vg_name, thinpool, lv_swap, return_output=True):
    """Swaps thinpool metadata to another LV
    :param vg_name: VG with both thinpool and swap LV in it
    :type vg_name: string
    :param thinpool: thinpool to swap metadata from
    :type thinpool: string
    :param lv_swap: LV to swap metadata to
    :type lv_swap: string
    :param return_output: should just errors or also retcode? (default True - both)
    :type return_output: bool
    :return: retcode, list of errors
    :rtype: tuple.
    """
    errors = []
    atomic_run(
        "Deactivating pool",
        vg_name=vg_name,
        lv_name=thinpool,
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap",
        vg_name=vg_name,
        lv_name=lv_swap,
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping metadata",
        vg_name=vg_name,
        lv_name=lv_swap,
        options=["-y", f"--thinpool {vg_name}/{thinpool} --poolmetadata"],
        command=lv_convert,
        errors=errors,
    )
    if return_output:
        if errors == []:
            return 0, errors
        return 1, errors
    else:
        return errors


def write_data(args, errors):
    """Function for writing persistent var files
    :param args: any arguments supplied to general function
    :type args: dict
    :param errors: list of errors
    :type errors: list
    :return: errors
    :rtype: list.
    """
    name = read_env("fmf_name")

    if "create_loopdev" in name:
        atomic_run(
            "Writing var %s" % args["name"].upper(),
            command=write_var,
            var={"%s" % args["name"].upper(): "/dev/%s" % args["name"]},
            errors=errors,
        )
        atomic_run(
            "Writing var %s_SIZE" % args["name"].upper(),
            command=write_var,
            var={"%s_SIZE" % args["name"].upper(): args["size"]},
            errors=errors,
        )

    if "create_vg" in name:
        atomic_run(
            "Writing var VG_NAME",
            command=write_var,
            var={"VG_NAME": args["vg_name"]},
            errors=errors,
        )

    if "create_lv" in name:
        atomic_run(
            "Writing var %s" % args["lv_name"].upper(),
            command=write_var,
            var={args["lv_name"].upper(): args["lv_name"]},
            errors=errors,
        )

    if "backup_metadata" in name:
        atomic_run(
            "Writing var METADATA_BACKUP",
            command=write_var,
            var={"METADATA_BACKUP": args["output"]},
            errors=errors,
        )
    return errors


def create_loopdev(**kwargs):
    """:param kwargs: all args taken by libsan.host.loopdev.create_loopdev
    :type kwargs: dict
    :return: return of libsan.host.loopdev.create_loopdev / None if fail
    :rtype: str / None
    """
    from os import path

    from libsan.host.linux import get_free_space
    from libsan.host.loopdev import create_loopdev

    possible_image_paths = read_env("fmf_possible_image_paths")
    image_path = None

    for possible_path in possible_image_paths:
        # check if the destination exists
        if not path.exists(possible_path):
            print("INFO: path '%s' does not exists, skipping." % possible_path)
            continue
        # comparing MB
        if (float(get_free_space(possible_path)) / 1024**2) > kwargs["size"]:
            image_path = possible_path
            break

    return create_loopdev(image_path=image_path, **kwargs) if image_path else None


def setup():
    """Basic function to merge it all together
    :return: errors
    :rtype: list.
    """
    errors = []
    class_instance = DMPD()
    args = get_dmpd_args(dmpd_object=class_instance)
    args.update(
        get_env_args(
            {
                "loop_name": str,
                "size": int,
                "pv_name": str,
                "vg_name": str,
                "lv_name": str,
                "lv_swap": str,
                "path": str,
                "options": list,
                "number_of_vols": int,
                "thinpool": str,
                "source_vg": str,
                "source_lv": str,
                "source_file": str,
                "cmd": str,
                "new_name": str,
            },
            read_var("FILE_NAMES"),
            {"loop_name": "name"},
        ),
    )

    command = args.pop("command")

    args["command"] = get_func_from_string(class_instance, command, local_functions=globals())
    atomic_run(errors=errors, **args)

    write_data(args, errors)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.vdo import VDO

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string
from stqe.host.persistent_vars import read_env, read_var, write_var
from stqe.host.vdo import (
    get_underlying_device,
    get_vdo_args,
    maximum_logical_size,
    minimum_slab_size,
    report_modify_difference,
    restart_modify,
)


def requires_restart_after_modification():
    return list(modification_args())


def modification_args(command=False):
    if command:
        return {
            "compression": "Compression",
            "deduplication": "Deduplication",
            "activate": "Activate",
            "deactivate": "Activate",
            "change_write_policy": "Configured write policy",
            "start": "Device mapper status",
            "stop": "Device mapper status",
            "grow": "size",
        }
    return {
        "ack_threads": "Acknowledgement threads",
        "bio_rotation_interval": "Bio rotation interval",
        "bio_threads": "Bio submission threads",
        "block_map_cache_size": "Block map cache size",
        "block_map_period": "Block map period",
        "cpu_threads": "CPU-work threads",
        "hash_zone_threads": "Hash zone threads",
        "logical_threads": "Logical threads",
        "max_discard_size": "Max discard size",
        "physical_threads": "Physical threads",
    }


def commands_changing_stats():
    return [
        "compression",
        "deduplication",
        "activate",
        "deactivate",
        "change_write_policy",
        "stop",
        "start",
        "grow",
    ]


def list_not_changing(**kwargs):
    from libsan.host.lvm import lv_create, lv_remove

    def _create_lvs(vg, count, errors):
        for i in range(count):
            atomic_run(
                "Creating LV lv%s" % i,
                command=lv_create,
                lv_name="lv%s" % i,
                vg_name=vg,
                options=["-l %s%%VG" % int(100 / count)],
                errors=errors,
            )
        print("INFO: All LVs created.")

    def _remove_lvs(vg, count, errors):
        for i in range(count):
            atomic_run(
                "Removing LV lv%s" % i,
                command=lv_remove,
                lv_name="lv%s" % i,
                vg_name=vg,
                errors=errors,
            )
        print("INFO: All LVs removed.")

    def _create_vdos(vdo, vg, count, errors):
        for i in range(count):
            atomic_run(
                "Creating VDO vdo%s" % i,
                command=vdo.create,
                name="vdo_list%s" % i,
                device=f"/dev/mapper/{vg}-lv{i}",
                slab_size=minimum_slab_size(f"/dev/mapper/{vg}-lv{i}", default_to_2g=False),
                errors=errors,
            )
        print("INFO: All VDOs created.")

    def _remove_vdos(vdo, count, errors):
        for i in range(count):
            atomic_run(
                "Removing VDO vdo%s" % i,
                command=vdo.remove,
                force=True,
                name="vdo_list%s" % i,
                errors=errors,
            )
        print("INFO: All VDOs removed.")

    errors = []

    vg = read_var("VG_NAME")
    _create_lvs(vg, kwargs["count"], errors)

    vdo = VDO(disable_check=True)
    _create_vdos(vdo, vg, kwargs["count"], errors)
    x = ""
    ret = 0
    vdo = VDO(disable_check=True)
    for i in range(kwargs["iterations"]):
        print("I: %s" % i)
        _, y = vdo.list(return_output=True)
        if i == 0:
            x = y
            continue
        if x != y:
            ret = 1
            e = "##### DIFFERENT LISTINGS, iteration %s #####\n" % i
            e += "X: %s\n" % x
            e += "#####\n"
            e += "Y: %s\n" % y
            errors.append(e)
            break
    out = "Checking listing passed %s iterations." % kwargs["iterations"] if errors == [] else "/n".join(errors)
    _remove_vdos(vdo, kwargs["count"], errors)
    _remove_lvs(vg, kwargs["count"], errors)
    print(out, str)
    return ret, out


def grow_physical(return_output=True):
    from libsan.host.lvm import lv_extend

    errors = []
    lv = read_var("LV_THIN")
    vg = read_var("VG_NAME")
    vdo = VDO(disable_check=True)
    min_slab_size = minimum_slab_size(f"/dev/mapper/{vg}-{lv}", default_to_2g=False)
    arguments = [
        {
            "message": "Creating VDO to be grown",
            "command": vdo.create,
            "name": "vdo_grow",
            "device": f"/dev/mapper/{vg}-{lv}",
            "force": True,
            "slab_size": min_slab_size,
        },
        {
            "message": "Growing LV under VDO to have extra physical size",
            "command": lv_extend,
            "vg_name": vg,
            "lv_name": lv,
            "options": ["-L +%s" % (str(int(min_slab_size[:-1]) * 3) + "M")],
        },
        {
            "message": "Growing physical size",
            "command": vdo.grow,
            "grow_type": "physical",
            "name": "vdo_grow",
        },
        {"message": "Removing VDO device", "command": vdo.remove, "name": "vdo_grow"}
        # TODO: Reduce the LV to original size
    ]
    ret = 0
    for argument in arguments:
        ret_cycle = atomic_run(errors=errors, **argument)
        if ret_cycle != 0:
            ret = 1
    return (ret, "/n".join(errors)) if return_output else ret


def call_func(args, vdo_object, errors):
    # slab size checking
    if "create" in read_env("fmf_name") and "success" in read_env("fmf_name") and "slab_size" not in args:
        args["slab_size"] = minimum_slab_size(args["device"])
        if args["slab_size"] == "2G":
            # 2G is default, no need to specify it
            args.pop("slab_size")

    if "slab_size" in args and args["slab_size"] == "minimum":
        args["slab_size"] = minimum_slab_size(args["device"], default_to_2g=False)

    if "logical_size" in args and args["logical_size"] == "maximum":
        args["logical_size"] = maximum_logical_size()

    if (
        "vdo_cli/modify" in read_env("fmf_name") or "vdo_cli/various_commands" in read_env("fmf_name")
    ) and "success" in read_env("fmf_name"):
        extra_args = {"name": args["name"]} if "name" in args else {}
        extra_args = {"conf_file": args["conf_file"]} if "conf_file" in args else extra_args
        _, args["status_before"] = atomic_run(
            "Getting VDO status after modifications",
            return_output=True,
            verbosity=False,
            command=vdo_object.status,
            errors=errors,
            **extra_args,
        )

    if "various_commands/remove/success" in read_env("fmf_name"):
        device = get_underlying_device(
            name=args["name"],
            conf_file=args["conf_file"] if "conf_file" in args else "/etc/vdoconf.yml",
        )
        if device:
            atomic_run(
                "Replacing var VDO_DEVICE with underlying device.",
                command=write_var,
                var={"VDO_DEVICE": device},
                errors=errors,
            )
    return args


def vdo_cli():
    errors = []
    vdo = VDO(disable_check=True)
    args = get_vdo_args(vdo_object=vdo)
    # some extra args for some setups
    args.update(
        get_env_args(
            {
                "path": str,
                "enable": None,
                "iterations": int,
                "count": int,
                "vg_name": None,
            }
        )
    )

    args = call_func(args, vdo, errors)
    if args is None:
        return errors
    status_before = args.pop("status_before") if "status_before" in args else ""

    command_string = args["command"]
    args["command"] = get_func_from_string(vdo, args.pop("command"), local_functions=globals())

    ret = atomic_run(errors=errors, **args)

    check_clean(
        ret=ret,
        vdo_object=vdo,
        args=args,
        errors=errors,
        status_before=status_before,
        command=command_string,
    )

    return errors


def check_clean(**kwargs):
    try:
        if read_env("fmf_requires_restart") is True:
            restart_modify(
                vdo_object=kwargs["vdo_object"],
                vdo_name=kwargs["args"]["name"],
                errors=kwargs["errors"],
            )
    except KeyError:
        pass
    if (
        ("vdo_cli/modify" in read_env("fmf_name") or kwargs["command"] in commands_changing_stats())
        and "success" in read_env("fmf_name")
        and not any([True for x in ["log_file", "conf_file", "verbose"] if x in read_env("fmf_name")])
    ):
        name = {"name": kwargs["args"]["name"]} if "name" in kwargs["args"] else {}
        _, status_after = atomic_run(
            "Getting VDO status after change",
            return_output=True,
            verbosity=False,
            command=kwargs["vdo_object"].status,
            errors=kwargs["errors"],
            **name,
        )
        changed_vars_name = []
        changed_args = []
        is_command = kwargs["command"] in commands_changing_stats()

        for arg in [*list(kwargs["args"]), kwargs["command"]]:
            if arg in modification_args(is_command):
                changed_vars_name.append(modification_args(is_command)[arg])
                changed_args.append(arg)

        for i, changed_var_name in enumerate(changed_vars_name):
            report_modify_difference(
                errors=kwargs["errors"],
                status_old=kwargs["status_before"],
                status_new=status_after,
                changed_var=changed_var_name,
                changed_argument=changed_args[i],
            )
    return


if __name__ == "__main__":
    errs = vdo_cli()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var


def disable_chap():
    errors = []
    target_iqn = read_var("TARGET_IQN")
    tpg = read_var("TPG")

    target = TargetCLI(path="/iscsi")

    atomic_run(
        "Disabling discovery_auth",
        enable=0,
        group="discovery_auth",
        command=target.set,
        errors=errors,
    )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run(
        "Disabling authentication",
        group="attribute",
        authentication=0,
        command=target.set,
        errors=errors,
    )

    for i in [
        "CHAP_MUTUAL_USERID",
        "CHAP_MUTUAL_PASSWORD",
        "CHAP_PASSWORD",
        "CHAP_USERID",
    ]:
        try:
            atomic_run(
                "Removing var: %s from /tmp" % i,
                var=i,
                command=clean_var,
                errors=errors,
            )
        except OSError:
            pass

    return errors


if __name__ == "__main__":
    errs = disable_chap()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def access_group_add_success():
    errors = []

    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID_2")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Adding initiator {} to AG {} with protocol {}.".format(init_id, ag_id, config["protocol"]),
            command=lsm.access_group_add,
            ag=ag_id,
            init=init_id,
            errors=errors,
        )

        atomic_run(
            "Removing initiator {} from AG {} with protocol {}.".format(init_id, ag_id, config["protocol"]),
            command=lsm.access_group_remove,
            ag=ag_id,
            init=init_id,
            force=True,
            errors=errors,
        )
    return errors


def access_group_add_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail adding initiator to AG without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_add,
            },
            {
                "message": "Trying to fail adding initiator to AG without AG with protocol %s" % config["protocol"],
                "command": lsm.access_group_add,
                "init": init_id,
            },
            {
                "message": "Trying to fail adding initiator to AG without init with protocol %s" % config["protocol"],
                "command": lsm.access_group_add,
                "ag": ag_id,
            },
            {
                "message": "Trying to fail adding initiator to AG with wrong init '-1' with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_add,
                "init": -1,
                "ag": ag_id,
            },
            {
                "message": "Trying to fail adding initiator to AG with NONEXISTENT AG with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_add,
                "init": init_id,
                "ag": "NONEXISTENT",
            },
            {
                "message": "Trying to fail adding initiator to AG with NONEXISTENT init with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_add,
                "init": "NONEXISTENT",
                "ag": ag_id,
            },
            {
                "message": "Trying to fail adding initiator to AG with AG name instead of AG ID with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_add,
                "init": init_id,
                "ag": ag_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def access_group_add_fail_no_state_change():
    errors = []

    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Adding initiator {} to AG {} to hit NO STATE CHANGE "
            "with protocol {}.".format(init_id, ag_id, config["protocol"]),
            command=lsm.access_group_add,
            ag=ag_id,
            init=init_id,
            expected_ret=4,
            errors=errors,
        )
    return errors


def access_group_add_fail_exists_initiator():
    errors = []

    ag_name_2 = read_var("LSM_AG_NAME_2")
    sys_id = read_var("LSM_SYS_ID")
    init_id_2 = read_var("LSM_INIT_ID_2")
    ag_id = read_var("LSM_AG_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Creating AG_2 {} with protocol {}.".format(ag_name_2, config["protocol"]),
            command=lsm.access_group_create,
            name=ag_name_2,
            init=init_id_2,
            sys=sys_id,
            return_output=True,
            errors=errors,
        )
        ag_id_2 = [line.split()[0].strip() for line in data.splitlines() if ag_name_2 in line][0]

        atomic_run(
            "Adding initiator {} to AG {} to hit EXISTS_INITIATOR "
            "with protocol {}.".format(init_id, ag_id, config["protocol"]),
            command=lsm.access_group_add,
            ag=ag_id,
            init=init_id_2,
            expected_ret=4,
            errors=errors,
        )

        atomic_run(
            "Deleting AG_2 {} with protocol {}.".format(ag_name_2, config["protocol"]),
            command=lsm.access_group_delete,
            ag=ag_id_2,
            force=True,
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = access_group_add_success()
    if int(environ["fmf_tier"]) == 2:
        errs = access_group_add_fail()
        errs += access_group_add_fail_no_state_change()
        errs += access_group_add_fail_exists_initiator()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import get_local_disk_data, yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_ident_led_success():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, disks = atomic_run(
            "Listing local disks before changing ident LED.",
            command=lsm.local_disk_list,
            return_output=True,
            script=True,
            errors=errors,
        )
        data_before = get_local_disk_data(disks)

        atomic_run(
            "Enabling ident LED of volume %s." % vol,
            command=lsm.volume_ident_led_on,
            vol=vol,
            errors=errors,
        )

        atomic_run(
            "Disabling ident LED of volume %s." % vol,
            command=lsm.volume_ident_led_off,
            vol=vol,
            errors=errors,
        )

        _, disks = atomic_run(
            "Listing local disks after changing ident LED.",
            command=lsm.local_disk_list,
            return_output=True,
            script=True,
            errors=errors,
        )
        data_after = get_local_disk_data(disks)

        if data_before != data_after:
            atomic_run(
                "Enabling ident LED to return it to previous state with protocol %s." % config["protocol"],
                command=lsm.volume_ident_led_on,
                vol=vol,
                errors=errors,
            )
            _, disks = atomic_run(
                "Listing local disks after returning ident LED state.",
                command=lsm.local_disk_list,
                return_output=True,
                script=True,
                errors=errors,
            )
            data_after = get_local_disk_data(disks)
            if data_before != data_after:
                msg = "FAIL: Could not return ident LED to previous state with protocol %s" % config["protocol"]
                print(msg)
                errors.append(msg)

    return errors


def volume_ident_led_on_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail enabling ident LED for volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_ident_led_on,
            },
            {
                "message": "Trying to fail enabling ident LED for volume with WRONG volume with protocol %s"
                % config["protocol"],
                "vol": "WRONG",
                "command": lsm.volume_ident_led_on,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_ident_led_off_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail disabling ident LED for volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_ident_led_off,
            },
            {
                "message": "Trying to fail disabling ident LED for volume with WRONG volume with protocol %s"
                % config["protocol"],
                "vol": "WRONG",
                "command": lsm.volume_ident_led_off,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_ident_led_offline():
    errors = []
    # TODO: Check changing ident LED when volume is disabled, create test
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_ident_led_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_ident_led_on_fail()
        errs += volume_ident_led_off_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.linux import os_arch

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def _is_wwpn(init):
    if len(init.split(":")) == 8:
        return True
    return False


def configure_lsm(protocols, username=None, password=None, target=None, **kwargs):  # noqaARG001
    print("INFO: Setting up libstoragemgmt for protocol %s." % protocols[0])
    arguments = locals()
    if "kwargs" in arguments:
        kwargs = arguments.pop("kwargs")
        for kwarg in kwargs:
            arguments[kwarg] = kwargs[kwarg]

    errors = []

    for argument in arguments:
        if arguments[argument] is None:
            continue

        var = "LSM_" + argument.upper()
        value = arguments[argument]
        # to write [val1, val2] instead of ['val1', 'val2']
        if isinstance(value, list):
            value = "[%s]" % ", ".join(value[:])
        atomic_run(
            "Writing var %s" % var,
            command=write_var,
            var={"%s" % var: value},
            errors=errors,
        )

    return errors


if __name__ == "__main__":
    # list of variables to save in /tmp as persistent vars
    arguments: dict = {
        "protocols": [],
        "username": None,
        "password": None,
        "target": None,
        "timeout": None,
        "pool_id": None,
        "pool_id_volume": None,
        "fs_size": None,
        "fs_export_path": None,
        "rw_host": None,
        "sys_id": None,
        "vol_size": None,
        "vol_size_2": None,
        "system_read_pct": None,
        "chap_in_pass": None,
        "chap_out_pass": None,
    }
    for argument in arguments:
        try:
            arguments[argument] = environ["fmf_" + argument]
        except KeyError:
            pass

    # these need to be arch specific
    arch = os_arch().replace("_", "-")  # '_' is invalid in IQN
    arguments_arch = {
        "fs_name": None,
        "pool_name": None,
        "fs_cloned_name": None,
        "fs_snap_name": None,
        "ag_name": None,
        "ag_name_2": None,
        "init_id": None,
        "init_id_chap": None,
        "init_id_2": None,
        "vol_name": None,
        "vol_name_2": None,
        "vol_rep_name": None,
        "chap_in_user": None,
        "chap_out_user": None,
    }  # type: dict
    for argument in arguments_arch:
        try:
            arguments_arch[argument] = environ["fmf_" + argument]
            if "init" in argument and _is_wwpn(arguments_arch[argument]):
                continue
            arguments_arch[argument] += "-%s" % arch
            # FS name cannot contain '-', INVALID_ARGUMENT(101)
            if any(x in argument for x in ["fs_name", "fs_cloned_name", "fs_snap_name"]):
                arguments_arch[argument] = arguments_arch[argument].replace("-", "_")
        except KeyError:
            pass
    arguments.update(arguments_arch)

    try:
        if not arguments["protocols"].startswith("["):
            raise TypeError  # noqaTRY301
        arguments["protocols"] = arguments["protocols"][1:-1].split(", ")
    except TypeError:
        print("ERROR: Protocols must be type list.")
        exit(1)

    errs = configure_lsm(**arguments)
    exit(parse_ret(errs))



#!/usr/bin/python


import os

from libsan.host.linux import (
    dist_ver,
    is_installed,
    is_service_running,
    service_start,
    service_status,
)
from libsan.host.mp import flush_all, mp_service_name, mp_start_service

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var


def start_multipath():
    errors = []
    multipath = None
    mp_socket_process = "multipathd.socket"
    if os.path.isfile("/tmp/START_MULTIPATH"):
        multipath = read_var("START_MULTIPATH")

    if not is_installed("device-mapper-multipath"):
        print("WARN: Skipping setup! Package device-mapper-multipath is not installed!")
        return errors

    if multipath is not None and multipath == 1:
        print("WARN: Skipping start of the services because they have not been running!")
        atomic_run(
            "Cleaning var START_MULTIPATH",
            command=clean_var,
            var="START_MULTIPATH",
            errors=errors,
        )

        return errors

    if float(dist_ver()) >= 8.0 and not is_service_running(mp_socket_process):
        ret = atomic_run(
            "Starting service %s" % mp_socket_process,
            service_name=mp_socket_process,
            command=service_start,
            errors=errors,
        )
        if ret != 5 and not is_service_running(mp_socket_process):
            msg = "FAIL: Service %s is not running" % mp_socket_process
            print(msg)
            errors.append(msg)
            return errors

    atomic_run("Starting service multipathd", command=mp_start_service, errors=errors)

    atomic_run("Flushing all", command=flush_all, errors=errors)

    if not is_service_running(mp_service_name()) and os.path.isfile("/etc/multipath.conf"):
        print("Printing %s status" % mp_service_name())
        service_status(mp_service_name())

        msg = "FAIL: Service %s is not running!" % mp_service_name()
        print(msg)
        errors.append(msg)
        return errors

    return errors


if __name__ == "__main__":
    errs = start_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python

from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var


def cleanup_crypt():
    name = read_var("STRATIS_DEVICE")
    errors = []

    atomic_run("Stopping crypt device %s" % name, cmd="cryptsetup close %s" % name, command=run, errors=errors)

    disks = read_var("CRYPT_DISKS")
    atomic_run(
        "Zeroing superblock on disks: %s" % disks,
        cmd="dd if=/dev/zero of=%s bs=1M count=1" % disks,
        command=run,
        errors=errors,
    )

    atomic_run("Cleaning var STRATIS_DEVICE", command=clean_var, var="STRATIS_DEVICE", errors=errors)

    atomic_run("Cleaning var CRYPT_DISKS", command=clean_var, var="CRYPT_DISKS", errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_crypt()
    exit(parse_ret(errs))



#!/usr/bin/python


import time

from libsan.host.cmdline import run
from libsan.host.linux import mkdir, mount, umount
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def stopped_pool_preserve_data():
    errors = []

    stratis = Stratis()
    pool_name = read_var("STRATIS_POOL")
    pool_uuid = read_var("STRATIS_POOL_UUID")
    fs_name = "fs_test"
    file_name = "test_file"
    mount_dir = "/mnt/pool_preserve_data"
    message = "this is pool preserve data test"
    stratis_fs_path = f"/dev/stratis/{pool_name}/{fs_name}"

    atomic_run(
        f"Creating filesystem: {fs_name}",
        pool_name=pool_name,
        fs_name=fs_name,
        command=stratis.fs_create,
        errors=errors,
    )

    time.sleep(2)

    atomic_run(f"Creating dir: {mount_dir}", new_dir=mount_dir, command=mkdir, errors=errors)

    atomic_run(
        f"Mounting filesystem: {fs_name}",
        device=stratis_fs_path,
        mountpoint=mount_dir,
        command=mount,
        errors=errors,
    )

    atomic_run(
        f"Writing data to file: {fs_name}",
        cmd=f"echo '{message}' > {mount_dir}/{file_name}",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Writing 1GB of random data to stratis fs",
        cmd=f"dd if=/dev/random of={mount_dir}/random_data bs=1M count=1000",
        command=run,
        errors=errors,
    )

    atomic_run(
        f"Trying to stop pool with mounted fs, this should fail: {fs_name}",
        pool_name=pool_name,
        command=stratis.pool_stop,
        expected_ret=1,
        expected_out=[
            "stratis_cli._errors.StratisCliEngineError: ERROR: low-level ioctl error due to nix error;"
            " header result:",
            "error: EBUSY: Device or resource busy",
        ],
        errors=errors,
    )

    atomic_run(f"Unmounting fs: {fs_name}", mountpoint=mount_dir, command=umount, errors=errors)

    atomic_run("Listing stratis filesystems", command=stratis.fs_list, errors=errors)

    atomic_run(
        "Trying to stop pool with unmounted fs",
        pool_name=pool_name,
        command=stratis.pool_stop,
        errors=errors,
    )

    atomic_run(
        "Starting pool to verify data",
        pool_uuid=pool_uuid,
        command=stratis.pool_start,
        errors=errors,
    )

    atomic_run("Listing stratis filesystems", command=stratis.fs_list, errors=errors)

    atomic_run(
        f"Mounting filesystem: {fs_name}",
        device=stratis_fs_path,
        mountpoint=mount_dir,
        command=mount,
        errors=errors,
    )

    _, data = atomic_run(
        "Check data on filesystem",
        return_output=True,
        cmd=f"cat {mount_dir}/{file_name}",
        command=run,
        errors=errors,
    )

    if message not in data:
        errors.append(f"FAIL: Could not find message: {message} in returned output!")

    atomic_run(
        "Writing 1GB of random data to stratis fs to verify fs is still usable",
        cmd=f"dd if=/dev/random of={mount_dir}/random_data2 bs=1M count=1000",
        command=run,
        errors=errors,
    )

    atomic_run(f"Unmounting fs: {fs_name}", mountpoint=mount_dir, command=umount, errors=errors)

    atomic_run(
        f"Destroying filesystem: {fs_name}",
        pool_name=pool_name,
        fs_name=fs_name,
        command=stratis.fs_destroy,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = stopped_pool_preserve_data()
    exit(parse_ret(errs))



#!/usr/bin/python


import secrets
import string
from os import environ
from random import randint

from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def remove_nones(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


def rand_string(length):
    """
    Creates random string of length passed as parameter.
    :param length: integer representing length of the string
    :return:
    None: if length is not integer
    string: with random characters
    """
    if not isinstance(length, int):
        return None

    alphabet = string.ascii_letters + string.digits + ".-+@_=:/[],~"  # rfc7143#section-6.1
    # Allowed in targetcli:
    # https://github.com/open-iscsi/configshell-fb/blob/b4923ee5591d8b980003150e0ba6ffe512d8c9da/configshell/shell.py#L116
    generated_string = "".join(secrets.choice(alphabet) for _ in range(length))

    return str(generated_string)


def configure_chap():
    errors = []
    target_iqn = read_var("TARGET_IQN")
    tpg = read_var("TPG")
    # supported fmf arguments
    args = {
        "random": False,
        "userid": None,
        "password": None,
        "mutual_userid": None,
        "mutual_password": None,
        "chap_ways": 1,
    }
    # Getting arguments from fmf
    for arg in args:
        try:
            if arg == "chap_ways":
                args[arg] = int(environ["fmf_%s" % arg])
                continue
            args[arg] = environ["fmf_%s" % arg]
        except KeyError:
            pass

    if args["random"]:  # if random is true, generate userid and password randomly
        if args["chap_ways"] == 2:
            args["mutual_userid"] = rand_string(randint(1, 255))
            args["mutual_password"] = rand_string(randint(1, 255))
        args["userid"] = rand_string(randint(1, 255))
        args["password"] = rand_string(randint(1, 255))
    for i in [
        "random",
        "chap_ways",
    ]:  # removing these args from dict because this dict is used as kwargs in atomic run
        args.__delitem__(i)

    target = TargetCLI()

    target.path = "/iscsi"

    atomic_run(
        "Enabling discovery_auth and setting userid and password",
        enable=1,
        group="discovery_auth",
        command=target.set,
        errors=errors,
        **remove_nones(args),
    )

    target.path = "/iscsi/" + target_iqn + "/" + tpg

    atomic_run(
        "Enabling authentication and generate node acls",
        group="attribute",
        authentication=1,
        generate_node_acls=1,
        command=target.set,
        errors=errors,
    )

    atomic_run(
        "Setting userid and password in auth group",
        group="auth",
        command=target.set,
        errors=errors,
        **remove_nones(args),
    )

    if args["mutual_userid"] is not None and args["mutual_password"] is not None:
        atomic_run(
            "Writing var CHAP_MUTUAL_USERID",
            command=write_var,
            var={"CHAP_MUTUAL_USERID": args["mutual_userid"]},
            errors=errors,
        )

        atomic_run(
            "Writing var CHAP_MUTUAL_PASSWORD",
            command=write_var,
            var={"CHAP_MUTUAL_PASSWORD": args["mutual_password"]},
            errors=errors,
        )

    atomic_run(
        "Writing var CHAP_USERID",
        command=write_var,
        var={"CHAP_USERID": args["userid"]},
        errors=errors,
    )

    atomic_run(
        "Writing var CHAP_PASSWORD",
        command=write_var,
        var={"CHAP_PASSWORD": args["password"]},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = configure_chap()
    exit(parse_ret(errs))



#!/usr/bin/python


import time

from libsan.host.cmdline import run
from libsan.host.linux import get_system_logs, mkdir, mkfs, mount, rmdir, umount
from libsan.host.lvm import lv_create, pv_remove, vg_create, vg_remove

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var

start = time.mktime(time.localtime())

error = []

device = read_var("LOOP1")
if not device:
    error.append("FAIL: Could not get the device from read_var")
    exit(parse_ret(error))
atomic_run(
    "the device is loop1",
    command=run,
    cmd="lsblk | grep %s" % device.split("/")[2],
    return_output=True,
    errors=error,
)

# SETUP
parted = [
    "parted -s %s mklabel MSDOS" % device,
    "parted -s %s mkpart pri 1 1248" % device,
    "parted -s %s mkpart ext 1249 3814" % device,
    "parted -s %s mkpart log 1249 1728" % device,
    "parted -s %s mkpart log 1729 1760" % device,
    "parted -s %s mkpart log 1761 2016" % device,
    "parted -s %s mkpart log 2017 2047" % device,
    "parted -s %s mkpart log 2048 2687" % device,
    "parted -s %s mkpart log 2688 3007" % device,
    "parted -s %s mkpart log 3008 3320" % device,
    "parted -s %s mkpart log 3321 3336" % device,
    "parted -s %s mkpart log 3337 3814" % device,
]
for cmd in parted:
    atomic_run("Running parted cmd '%s'" % cmd, command=run, cmd=cmd, errors=error)
atomic_run("Running udevadm trigger", command=run, cmd="udevadm trigger", errors=error)
atomic_run("Running lsblk cmd", command=run, cmd="lsblk | grep loop", errors=error)
atomic_run(
    "Creating VG 'vgtest'",
    command=vg_create,
    vg_name="vgtest",
    pv_name="%s" % device + (" %s" % device).join(["p" + str(x) for x in [1, 5, 8, 9, 10, 11, 13]]),
    errors=error,
)
atomic_run(
    "Creating LV",
    command=lv_create,
    lv_name="testvol",
    vg_name="vgtest",
    options=["-l 100%FREE", "-y"],
    errors=error,
)
atomic_run(
    "Creating filesystem",
    command=mkfs,
    device_name="/dev/mapper/vgtest-testvol",
    fs_type="xfs",
    errors=error,
)
atomic_run("Creating mount dir", command=mkdir, new_dir="/tmp/test_mount", errors=error)
atomic_run(
    "Mounting to /tmp/test_mount",
    command=mount,
    device="/dev/mapper/vgtest-testvol",
    mountpoint="/tmp/test_mount",
    errors=error,
)
# TEST

atomic_run(
    "Creating test file",
    command=run,
    cmd="dd if=/dev/urandom of=/tmp/test_mount/test_file bs=3K count=1128966 status=progress",
    expected_ret=1,
    expected_out="No space left on device",
    errors=error,
)
atomic_run("Syncing mount point", command=run, cmd="sync /tmp/test_mount", errors=error)
atomic_run(
    "Removing test file",
    command=run,
    cmd="rm -f /tmp/test_mount/test_file",
    errors=error,
)
atomic_run("Syncing mount point", command=run, cmd="sync /tmp/test_mount", errors=error)
atomic_run(
    "Sending trim",
    command=run,
    cmd="fstrim /tmp/test_mount",
    expected_out=read_env("fmf_expected_fstrim_out"),
    expected_ret=read_env("fmf_expected_fstrim_ret"),
    errors=error,
)

ret, syslogs = atomic_run(
    message="Checking logs for errors.",
    command=get_system_logs,
    since="-%s" % int(time.mktime(time.localtime()) - start),
    return_output=True,
    errors=error,
)
sys_error = "attempt to access beyond end of device"
if sys_error in syslogs:
    error.append("FAIL: Found error '%s' in syslog." % sys_error)

# CLEANUP
atomic_run(
    "Umounting/tmp/test_mount",
    command=umount,
    mountpoint="/tmp/test_mount",
    errors=error,
)
atomic_run("Removing mount dir", command=rmdir, dir_name="/tmp/test_mount", errors=error)
atomic_run(
    "Removing VG 'vgtest'",
    command=vg_remove,
    vg_name="vgtest",
    force=True,
    errors=error,
)
for pv in [" %sp" % device + str(x) for x in [1, 5, 8, 9, 10, 11, 13]]:
    atomic_run("Removing PV %s" % pv, command=pv_remove, pv_name=pv, errors=error)
atomic_run(
    "Removing partitioning table",
    command=run,
    cmd="dd if=/dev/zero of=%s bs=512 count=1" % device,
    errors=error,
)
atomic_run(
    "Reloading partitioning table",
    command=run,
    cmd="partprobe %s" % device,
    errors=error,
)

exit(parse_ret(error))



#!/usr/bin/python


from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def unmask_volume():
    print("INFO: Unmasking volume.")
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    atomic_run(
        f"Unmasking volume {vol_id} to access group {ag_id}",
        command=lsm.volume_unmask,
        vol=vol_id,
        ag=ag_id,
        errors=errors,
    )
    return errors


if __name__ == "__main__":
    errs = unmask_volume()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.linux import mkdir, mount, umount
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env


def pool_out_of_space():
    errors = []

    pool_name = "pool_out_of_space2"
    fs_size = 512
    loopdev_size = 10400
    fs_name = "fs_fail"
    mount_dir = "/mnt/pool_out_of_space"
    stratis_fs_path = f"/dev/stratis/{pool_name}/fs1"
    no_overprovision = None
    stratis = Stratis()

    try:
        no_overprovision = read_env("fmf_pool_nooverprovision")
    except KeyError:
        pass

    loop_path = create_loopdev(size=loopdev_size)

    if not no_overprovision:
        atomic_run(
            f"Creating pool: {pool_name}",
            pool_name=pool_name,
            blockdevs=loop_path,
            command=stratis.pool_create,
            errors=errors,
        )

    else:
        atomic_run(
            f"Creating pool: {pool_name}",
            pool_name=pool_name,
            blockdevs=loop_path,
            no_overprovision=True,
            command=stratis.pool_create,
            errors=errors,
        )

    # for an approximate result in MiB, divide loopdev_size value by 1.049. Then divide
    # result with fs size to get how many fs we should create in order to fill pool.
    # pool always takes ~500MiB when created, so this formula should execute 1 or 2
    # fs create commands that will fail.
    num_of_filesystems = int((loopdev_size / 1.049) / fs_size)
    for fs in range(num_of_filesystems):
        stratis.fs_create(
            pool_name=pool_name,
            fs_name=f"fs{fs}",
            fs_size=f"{fs_size}MiB",
        )

    atomic_run(
        "Creating FS, this should fail because doesn't have enough space",
        expected_out=["Error: Command failed:", "No space left on device"],
        expected_ret=1,
        pool_name=pool_name,
        fs_name=fs_name,
        command=stratis.fs_create,
        errors=errors,
    )

    atomic_run(f"Creating dir: {mount_dir}", new_dir=mount_dir, command=mkdir, errors=errors)

    atomic_run(
        f"Mounting filesystem: {fs_name}",
        device=stratis_fs_path,
        mountpoint=mount_dir,
        command=mount,
        errors=errors,
    )

    # at this point fs snapshot should fail as well because pool has not enough space
    atomic_run(
        "Creating FS snapshot, this should fail because doesn't have enough space",
        expected_ret=1,
        # expected_out="",
        pool_name=pool_name,
        snapshot_name="snap1",
        origin_name="fs1",
        command=stratis.fs_snapshot,
        errors=errors,
    )

    atomic_run("Unmounting filesystem", mountpoint=mount_dir, command=umount, errors=errors)

    for fs in range(num_of_filesystems):
        atomic_run(
            f"Destroying filesystem: fs{fs}",
            pool_name=pool_name,
            fs_name=f"fs{fs}",
            command=stratis.fs_destroy,
            errors=errors,
        )

    atomic_run(
        f"Destroying pool: {pool_name}",
        pool_name=pool_name,
        command=stratis.pool_destroy,
        errors=errors,
    )

    atomic_run("Removing loopdev", name=loop_path, command=delete_loopdev, errors=errors)

    return errors


if __name__ == "__main__":
    errs = pool_out_of_space()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config


def plugin_info_success():
    errors = []
    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)
        atomic_run(
            "Using LSM to get plugin info with protocol %s" % config["protocol"],
            command=lsm.plugin_info,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = plugin_info_success()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_replicate_range_success():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        rep_types = ["CLONE", "COPY"]
        if "ontap" in config["protocol"]:
            rep_types = ["CLONE"]  # copy is not supported on ontap yet

        for rep_type in rep_types:
            atomic_run(
                "Replicating volume %s to volume %s with type %s with protocol %s."
                % (src_vol, dst_vol, rep_type, config["protocol"]),
                command=lsm.volume_replicate_range,
                src_vol=src_vol,
                dst_vol=dst_vol,
                rep_type=rep_type,
                src_start=0,
                dst_start=0,
                count=100,
                force=True,
                errors=errors,
            )

            atomic_run(
                f"Removing dependency between replicated volumes {src_vol} and {dst_vol}",
                command=lsm.volume_dependants_rm,
                vol=src_vol,
                errors=errors,
            )

            atomic_run(
                "Replicating volume %s to volume %s with type %s with repeated arguments with protocol %s."
                % (src_vol, dst_vol, rep_type, config["protocol"]),
                command=lsm.volume_replicate_range,
                src_vol=src_vol,
                dst_vol=dst_vol,
                rep_type=rep_type,
                src_start=[0, 100, -100],
                dst_start=[0, 200, -100],
                count=[100, 1, -100],
                force=True,
                errors=errors,
            )

            atomic_run(
                f"Removing dependency between replicated volumes {src_vol} and {dst_vol}",
                command=lsm.volume_dependants_rm,
                vol=src_vol,
                errors=errors,
            )
    return errors


def volume_replicate_range_fail():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail replicating volume range without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without source volume with protocol %s"
                % config["protocol"],
                "dst_vol": dst_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without destination volume with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without replication type volume with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "dst_vol": dst_vol,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without source start with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": dst_vol,
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without destination start with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "src_start": 0,
                "dst_vol": dst_vol,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range without block count with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "src_start": 0,
                "dst_start": 0,
                "dst_vol": dst_vol,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong source volume with protocol %s"
                % config["protocol"],
                "src_vol": "WRONG",
                "dst_vol": dst_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong destination volume with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": "WRONG",
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong source start with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": dst_vol,
                "src_start": "WRONG",
                "dst_start": 0,
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong destination start with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": dst_vol,
                "src_start": 0,
                "dst_start": "WRONG",
                "count": 100,
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong block count with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": dst_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": "WRONG",
                "rep_type": rep_type,
                "command": lsm.volume_replicate_range,
            },
            {
                "message": "Trying to fail replicating volume range with wrong replication type with protocol %s"
                % config["protocol"],
                "src_vol": src_vol,
                "dst_vol": dst_vol,
                "src_start": 0,
                "dst_start": 0,
                "count": 100,
                "rep_type": "WRONG",
                "command": lsm.volume_replicate_range,
            },
        ]
        for argument in arguments:
            ret, _ = atomic_run(expected_ret=2, errors=errors, return_output=True, **argument)
            if ret == 0:
                atomic_run(
                    f"Removing dependency between replicated volumes {src_vol} and {dst_vol} after not failing",
                    command=lsm.volume_dependants_rm,
                    vol=src_vol,
                    errors=errors,
                )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_replicate_range_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_replicate_range_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret


def saveconfig_create():
    errors = []
    target = TargetCLI(path="")

    args = {"savefile": None}
    try:
        args["savefile"] = environ["fmf_savefile_name"]
    except KeyError:
        pass

    atomic_run("Creating saveconfig", command=target.saveconfig, errors=errors, **target.remove_nones(args))

    return errors


if __name__ == "__main__":
    errs = saveconfig_create()
    exit(parse_ret(errs))



#!/usr/bin/python


from time import sleep

from libsan.host.fcoe import unconfigure_soft_fcoe

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var


def cleanup_fcoe():
    print("INFO: Cleaning up soft FCoE.")
    errors = []

    print("Waiting for data stream to settle before cleaning up FCoE.")
    sleep(5)

    atomic_run("Cleaning up FCoE", command=unconfigure_soft_fcoe, errors=errors)

    atomic_run("Cleaning var VDO_DEVICE", command=clean_var, var="VDO_DEVICE", errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_fcoe()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var


def restart_pool():
    errors = []
    stratis = Stratis()
    pool_name = None
    pool_uuid = None
    unlock_method = None

    try:
        pool_name = read_env("fmf_pool_name")
    except KeyError:
        pass
    if not pool_name:
        try:
            pool_name = read_var("STRATIS_POOL")
            pool_uuid = read_var("STRATIS_POOL_UUID")
        except KeyError:
            pass
    if not pool_uuid:
        pool_uuid = stratis.get_pool_uuid(pool_name=pool_name)
    try:
        unlock_method = read_env("fmf_unlock_method")
    except KeyError:
        pass

    atomic_run(
        "Stopping stratis pool %s." % pool_name,
        command=stratis.pool_stop,
        pool_name=pool_name,
        errors=errors,
    )

    atomic_run(
        "Starting stratis pool %s." % pool_name,
        command=stratis.pool_start,
        unlock_method=unlock_method,
        pool_uuid=pool_uuid,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = restart_pool()
    exit(parse_ret(errs))



import os

import nox


@nox.session(python=["3.6", "3.9", "3.11"])
def tests(session: nox.Session) -> None:
    """Runs pytest"""
    var = "LIBSAN_VERSION"
    if var in os.environ:
        session.install(f"libsan=={os.environ[var]}")
    session.install("-e", ".")
    session.install("pytest", "pytest-cov")
    session.run(
        "pytest",
        "--cov=stqe",
        "--cov-config",
        "pyproject.toml",
        "--cov-report=",
        *session.posargs,
        env={"COVERAGE_FILE": f".coverage.{session.python}"},
    )
    session.notify("coverage")


@nox.session
def coverage(session) -> None:
    """Coverage analysis"""
    session.install("coverage[toml]")
    session.run("coverage", "combine")
    session.run("coverage", "report")
    session.run("coverage", "erase")


@nox.session
def stqe_test(session) -> None:
    """Test stqe-test cli"""
    var = "LIBSAN_VERSION"
    if var in os.environ:
        session.install(f"libsan=={os.environ[var]}")
    session.install("-e", ".")
    session.run("stqe-test", "run", "--fmf", "--norun", "--path", "lvm/device_mapper_persistent_data/thin")
    session.run("stqe-test", "run", "--fmf", "--norun", "-f", "component:lvm")
    session.run("stqe-test", "list", "--fmf", "--path", "lvm/device_mapper_persistent_data/", "tests")



#!/usr/bin/python


from libsan.host.loopdev import delete_loopdev

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def loopdev_delete():
    errors = []

    name = read_var("TARGETD_LOOPDEV_NAME")

    atomic_run("Deleting loopdev %s" % name, command=delete_loopdev, name=name, errors=errors)

    return errors


if __name__ == "__main__":
    errs = loopdev_delete()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def setup_crypt():
    disk = read_var("VDO_DEVICE")
    print("INFO: Setting up dm-crypt on device %s." % disk)
    errors = []

    name = "vdo_crypted"

    atomic_run(
        message="Creating key",
        command=run,
        cmd="echo -e  'y\\n'| ssh-keygen -f key -N redhat",
        errors=errors,
    )

    atomic_run(
        message="Encrypting disk with dm-crypt/LUKS.",
        command=run,
        cmd="cryptsetup luksFormat %s key --batch-mode" % disk,
        errors=errors,
    )

    atomic_run(
        message="Opening encrypted disk.",
        command=run,
        cmd=f"cryptsetup luksOpen {disk} {name} -d key",
        errors=errors,
    )

    atomic_run(
        "Writing var VDO_DEVICE",
        command=write_var,
        var={"VDO_DEVICE": "/dev/mapper/%s" % name},
        errors=errors,
    )

    atomic_run(
        "Writing var CRYPTED_DISK",
        command=write_var,
        var={"CRYPTED_DISK": "%s" % disk},
        errors=errors,
    )

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_crypt()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_env, read_var, write_var


def destroy_pool():
    errors = []
    stratis = Stratis()

    id = ""
    try:
        id = "_" + str(read_env("fmf_pool_id"))
    except KeyError:
        pass
    pool_name = read_var("STRATIS_POOL%s" % id)

    if "single" in read_env("fmf_name"):
        available_devices = read_var("STRATIS_AVAILABLE_DEVICES")
        write_var({"STRATIS_DEVICE": available_devices})
        free = read_var("STRATIS_FREE")
        if free:
            clean_var("STRATIS_FREE")

    atomic_run(
        "Destroying stratis pool %s." % pool_name,
        command=stratis.pool_destroy,
        pool_name=pool_name,
        errors=errors,
    )

    atomic_run(
        "Cleaning var STRATIS_POOL%s" % id,
        command=clean_var,
        var="STRATIS_POOL%s" % id,
        errors=errors,
    )

    atomic_run(
        "Cleaning var STRATIS_POOL_UUID",
        command=clean_var,
        var="STRATIS_POOL_UUID",
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = destroy_pool()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.linux import wait_udev

from stqe.host.atomic_run import atomic_run, parse_ret


def cleanup_multipath():
    print("INFO: Removing multipath.")
    errors = []

    _, multipath = atomic_run(
        message="Getting mpath device",
        command=run,
        return_output=True,
        cmd="multipath -l -v 1",
        errors=errors,
    )

    cmd = "-F"
    if multipath is not None:
        cmd = "-f %s" % multipath
    for mpath in cmd.splitlines():
        if "-" not in mpath:
            mpath = "-f " + mpath
        atomic_run(
            message="Removing mpath device %s" % multipath,
            command=run,
            cmd="multipath %s" % mpath,
            errors=errors,
        )

    # wait for udevadm to settle
    wait_udev()

    return errors


if __name__ == "__main__":
    errs = cleanup_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.dmpd import DMPD

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string
from stqe.host.lvm import get_dmpd_args


def dmpd_cli():
    errors = []
    dmpd = DMPD()
    args = get_dmpd_args(dmpd_object=dmpd)
    args.update(get_env_args({"source_vg": str, "source_lv": str, "source_file": str}))
    if args["command"] == "thin_trim":
        args["data_dev"] = f'{args["data_dev"]}_tdata'
    args["command"] = get_func_from_string(dmpd, args.pop("command"), local_functions=globals())

    atomic_run(errors=errors, **args)

    return errors


if __name__ == "__main__":
    errs = dmpd_cli()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_snap_restore_success():
    errors = []

    snap_id = read_var("LSM_FS_SNAP_ID")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Restoring FS {} from snapshot {} with protocol {}.".format(fs_id, snap_id, config["protocol"]),
            command=lsm.fs_snap_restore,
            fs=fs_id,
            snap=snap_id,
            force=True,
            errors=errors,
        )
        # FIXME: Add test with 'lsm_file' and 'fileas'
    return errors


def fs_snap_restore_fail():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    snap_id = read_var("LSM_FS_SNAP_ID")
    snap_name = read_var("LSM_FS_SNAP_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail restoring FS from snapshot without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_restore,
            },
            {
                "message": "Trying to fail restoring FS from snapshot without 'snap' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_restore,
                "fs": fs_id,
            },
            {
                "message": "Trying to fail restoring FS from snapshot without 'fs' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_restore,
                "snap": snap_id,
            },
            {
                "message": "Trying to fail restoring FS from snapshot with wrong 'snap' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_restore,
                "fs": fs_id,
                "snap": "wrong",
            },
            {
                "message": "Trying to fail restoring FS from snapshot with wrong 'fs' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_restore,
                "fs": "wrong",
                "snap": snap_id,
            },
            {
                "message": "Trying to fail restoring FS from snapshot by giving 'snap' name "
                "instead of ID with protocol %s" % config["protocol"],
                "command": lsm.fs_snap_restore,
                "snap": snap_name,
                "fs": fs_id,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_snap_restore_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_snap_restore_fail()
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
# Author: Bruno Goncalves   <bgoncalv@redhat.com>


import argparse
import os
import re
import subprocess
import sys

from libsan import sanmgmt
from libsan.host import dt, linux, mp, scsi
from libsan.host.cmdline import run

import stqe.host.logchecker as log
from stqe.host import nfs_lock

LOCK_TYPE = "shared"

clean_up_info: dict = {}

lock_obj = None


def _print(string):
    module_name = __name__
    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)

    # Append time information to command
    date = 'date "+%Y-%m-%d %H:%M:%S"'
    p = subprocess.Popen(date, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    stdout, _ = p.communicate()
    stdout = stdout.decode("ascii", "ignore")
    stdout = stdout.rstrip("\n")

    print(f"[{stdout}] {string}")

    sys.stdout.flush()
    if "FATAL:" in string:
        raise RuntimeError(string)
    return


def _clean_up():
    """ """
    global LOCK_TYPE, clean_up_info, lock_obj

    if not clean_up_info:
        return True

    mpath_names = clean_up_info["mpath_names"]
    if mpath_names:
        for mpath in mpath_names:
            mp.remove_mpath(mpath)
        clean_up_info["mpath_names"] = []

    wwids = clean_up_info["wwids"]
    if wwids:
        for wwid in wwids:
            scsi_ids = scsi.scsi_ids_of_wwid(wwid)
            if not scsi_ids:
                continue
            for scsi_id in scsi_ids:
                scsi_disk = scsi.get_scsi_disk_name(scsi_id)
                _print("INFO: Deleting disk: %s" % scsi_disk)
                scsi.delete_disk(scsi_disk)
            mp.mpath_conf_remove("/blacklist_exception/wwid", wwid)
        clean_up_info["wwids"] = []

    _print("INFO: All disks removed from the server")
    obj_sanmgmt = clean_up_info["obj_sanmgmt"]
    lun_names = clean_up_info["lun_names"]
    if lun_names:
        for lun_name in lun_names:
            _print("INFO: Deleting %s on the array..." % lun_name)
            if not obj_sanmgmt.lun_remove(lun_name):
                _print("WARN: Could not remove TMP LUN: %s" % lun_name)
    if lock_obj:
        lock_obj.release_lock(LOCK_TYPE)
    _print("INFO: Cleaned")
    return True


def execute_test(mpath_name, lun_count, lun_size):
    global LOCK_TYPE, clean_up_info, lock_obj

    _print("INFO: Using %s as base device" % mpath_name)
    _print("INFO: Will create %d new LUNs for this test" % lun_count)

    mp_info = mp.multipath_query_all(mpath_name)
    if not mp_info:
        _print("FAIL: Got NULL from query_mp_info(%s)" % mpath_name)
        return False

    obj_sanmgmt = sanmgmt.create_sanmgmt_for_mpath(mpath_name)
    if not obj_sanmgmt:
        _print("FAIL: Got NULL from create_sanmgmt_for_mpath(%s)" % mpath_name)
        return False

    clean_up_info["obj_sanmgmt"] = obj_sanmgmt

    scsi_disks = mp_info["disk"]

    scsi_host_ids = []
    for scsi_disk in scsi_disks:
        host_id = scsi.scsi_host_of_scsi_name(scsi_disk)
        if host_id and host_id not in scsi_host_ids:
            scsi_host_ids.append(host_id)

    clean_up_info["lun_names"] = []
    clean_up_info["wwids"] = []
    clean_up_info["mpath_names"] = []

    lock_obj = nfs_lock.setup_nfs_lock_for_mpath(mpath_name)
    if lock_obj:
        _print("INFO: Requesting NFS lock (%s)..." % LOCK_TYPE)
        if not lock_obj.request_lock(LOCK_TYPE):
            _print("FAIL: Could not request NFS lock")
            return False

        _print("INFO: Waiting for NFS lock...")
        if not lock_obj.get_lock():
            _print("FAIL: Give up waiting for lock")
            sys.exit(1)
        _print("INFO: Good, got NFS lock. Start testing...")

    for lun_cnt in range(1, lun_count + 1):
        lun_name = obj_sanmgmt.lun_create_and_map(size=lun_size)
        if not lun_name:
            _print("FAIL: Could not create LUN %d for add_new_lun test" % lun_cnt)
            _clean_up()
            return False
        clean_up_info["lun_names"].append(lun_name)

    _print("INFO: All LUNs successfully created")

    # need to rescan hosts
    for host in scsi_host_ids:
        scsi.rescan_host(host)
    # if adding more luns we should wait more time
    linux.sleep(6 * lun_count)
    linux.wait_udev()

    if "lun_names" not in clean_up_info:
        _print("FAIL: No LUNs created for test")
        _clean_up()
        return False

    for lun_name in clean_up_info["lun_names"]:
        scsi_ids = obj_sanmgmt.scsi_id_of_lun(lun_name=lun_name)
        if not scsi_ids:
            _print("FAIL: Failed to find out the SCSI ID for newly mapped LUN %s" % lun_name)
            run("ls /dev/sd*")
            print(scsi.query_all_scsi_disks())
            _clean_up()
            return False

        _print("INFO: New SCSI disks created:")
        for _ids in scsi_ids:
            _print(f"    {_ids} - {scsi.get_scsi_disk_name(_ids)}")

        # All disks should have the same wwid
        scsi_disk = scsi.get_scsi_disk_name(scsi_ids[0])
        wwid = scsi.wwid_of_disk(scsi_disk)
        if not wwid:
            _print("FAIL: Could not find WWID for %s" % scsi_disk)
            _clean_up()
            return False

        clean_up_info["wwids"].append(wwid)

        _print("INFO: New disks have WWID: %s" % wwid)
        if not mp.mpath_conf_set("/blacklist_exceptions/wwid[last()+1]", wwid):
            _print("FAIL: Cannot enable multipath for WWID: %s" % wwid)
            _clean_up()
            return False

        mp.multipath_reload_conf()
        linux.wait_udev()

        new_mpath = mp.mpath_name_of_wwid(wwid)
        if not new_mpath:
            _print("FAIL: Could not find mpath for WWID %s" % wwid)
            _clean_up()
            return False
        clean_up_info["mpath_names"].append(new_mpath)

        _print("INFO: Multipath enabled on tmp LUN: %s" % new_mpath)

        new_mp_info = mp.multipath_query_all(new_mpath)
        if not new_mp_info:
            _print("FAIL: Could not query mp info for %s" % new_mpath)
            _clean_up()
            return False

        lun_info = obj_sanmgmt.lun_info(lun_name)

        all_scsi_ids_info = scsi.query_all_scsi_disks()
        array_size = int(lun_info["size"])
        for scsi_id in scsi_ids:
            disk_size = int(all_scsi_ids_info[scsi_id]["size"])
            if all_scsi_ids_info[scsi_id]["vendor"] == "EQLOGIC":
                tolerance = int(array_size * 0.01)
                # check if disk is within tolerance level
                if (disk_size // array_size) < tolerance:
                    array_size = disk_size
            if disk_size != array_size:
                scsi_name = scsi.get_scsi_disk_name(scsi_id)
                _print(
                    "FAIL: newly created Disk: %s is holding size: '%s', while on storage array it is: '%s'"
                    % (scsi_name, disk_size, array_size)
                )
                _clean_up()
                return False

        # Check if mpath reported size correctly
        mp_size = new_mp_info["size_bytes"]
        if int(mp_size) != int(lun_info["size"]):
            _print(
                "FAIL: newly created mpath: %s is holding size: %s, while on storage array it is: %s"
                % (new_mpath, mp_size, int(lun_info["size"]))
            )
            _clean_up()
            return False

    # end lun_names for
    _print("INFO: All LUNs successfully found by host")

    # Run DT in all LUNs
    dt_pid_2_info = {}
    for mpath_name in clean_up_info["mpath_names"]:
        dt_log_file = "/tmp/dt_log_%s" % mpath_name
        pid = dt.dt_stress_background(of="/dev/mapper/%s" % mpath_name, log_file=dt_log_file, verbose=0)
        if not pid:
            _print("FAIL: Could not start DT I/O stress on %s" % mpath_name)
            _clean_up()
            sys.exit(1)
        dt_pid_2_info[pid] = {"log": dt_log_file, "exit_code": None}

    for dt_pid in dt_pid_2_info:
        # option for non blocking, in this case we will block. os.WNOHANG
        _, exit_status = os.waitpid(dt_pid, 0)
        dt_pid_2_info[dt_pid]["exit_code"] = exit_status

    dt_fail = False
    for dt_pid in dt_pid_2_info:
        if dt_pid_2_info[dt_pid]["exit_code"] != 0:
            _print(
                "FAIL: DT I/O stress PID {} failed with quit code {}".format(dt_pid, dt_pid_2_info[dt_pid]["exit_code"])
            )
            run("cat %s" % dt_pid_2_info[dt_pid]["log"])
            dt_fail = True

    if not dt_fail:
        _print("INFO: All I/O stress tests PASS")

    _clean_up()
    if not log.check_all() or dt_fail:
        _print("FAIL: detected error on logchecker or IO error")
        return False

    _print("PASS: All I/O stress tests")
    return True


def main():
    """ """
    pass_retcode = 0
    fail_retcode = 1

    parser = argparse.ArgumentParser(description="add new num")
    parser.add_argument(
        "--mpath-name",
        "-m",
        required=False,
        dest="mpath_name",
        help="Name of multipath device to use (We use it to create new device based on it)",
        metavar="mpath",
    )
    parser.add_argument(
        "--lun-count",
        required=False,
        dest="lun_count",
        default=1,
        help="How many LUN should be created",
        type=int,
        metavar="count",
    )
    parser.add_argument(
        "--lun-size",
        required=False,
        dest="lun_size",
        default="2GiB",
        help="What size should be the new LUN",
        metavar="size. Eg: 2GiB",
    )

    args = parser.parse_args()

    mpath_name_list = []
    if args.mpath_name:
        mpath_name_list.append(args.mpath_name)
    lun_count = args.lun_count
    lun_size = args.lun_size

    # If not mpath was specified search for them on the system
    if not mpath_name_list:
        choosen_devs = sanmgmt.choose_mpaths()
        if not choosen_devs:
            _print("FAIL: Could not find any multipath device to use as base LUN")
            sys.exit(fail_retcode)
        mpath_name_list = list(choosen_devs.keys())

    error = 0
    for mpath_name in mpath_name_list:
        if not execute_test(mpath_name, lun_count, lun_size):
            error += 1

    if error:
        sys.exit(fail_retcode)
    sys.exit(pass_retcode)


main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import get_data_from_script_output, yield_lsm_config
from stqe.host.persistent_vars import read_var


def system_read_cache_pct_update_success():
    errors = []

    sys = read_var("LSM_SYS_ID")

    read_pct = int(read_var("LSM_SYSTEM_READ_PCT"))

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Listing systems to backup read percentage",
            command=lsm.list,
            lsm_type="SYSTEMS",
            return_output=True,
            script=True,
            errors=errors,
        )
        pct_before = get_data_from_script_output(data)[sys]["Read Cache Percentage"]

        atomic_run(
            "Changing system read pct to {} with protocol {}".format(read_pct, config["protocol"]),
            command=lsm.system_read_cache_pct_update,
            sys=sys,
            read_pct=read_pct,
            errors=errors,
        )

        _, data = atomic_run(
            "Listing systems to check updated read percentage",
            command=lsm.list,
            lsm_type="SYSTEMS",
            return_output=True,
            script=True,
            errors=errors,
        )
        pct_after = int(get_data_from_script_output(data)[sys]["Read Cache Percentage"])
        if pct_after != read_pct:
            msg = f"FAIL: System cache read percentage did not change, should be {read_pct}, got {pct_after}"
            print(msg)
            errors.append(msg)

        atomic_run(
            "Returning system read pct to former value with protocol %s" % config["protocol"],
            command=lsm.system_read_cache_pct_update,
            sys=sys,
            read_pct=pct_before,
            errors=errors,
        )

    return errors


def system_read_cache_pct_update_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail changing system read pct without path with protocol %s" % config["protocol"],
                "command": lsm.system_read_cache_pct_update,
            },
            {
                "message": "Trying to fail changing system read pct with wrong system with protocol %s"
                % config["protocol"],
                "command": lsm.system_read_cache_pct_update,
                "sys": "WRONG",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = system_read_cache_pct_update_success()
    if int(environ["fmf_tier"]) == 2:
        errs = system_read_cache_pct_update_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


import os
from time import sleep

from libsan.host import linux, targetd
from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_ssl_func_test():
    errors = []

    password = os.environ["fmf_ssl_password"]

    atomic_run(
        "Starting targetd SSL NO KEY test",
        enable=True,
        command=targetd.ssl_change,
        errors=errors,
    )

    atomic_run(
        "Adding password to the config",
        password=password,
        command=targetd.set_password,
        errors=errors,
    )

    atomic_run(
        "Trying to start targetd without SSL key, it should fail!",
        service_name="targetd",
        command=linux.service_restart,
        errors=errors,
    )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run(
        "Getting targetd service status",
        retcode=3,
        command=targetd.targetd_status,
        errors=errors,
    )

    atomic_run("Removing password from config", command=targetd.set_password, errors=errors)

    cmd = "systemctl status targetd -l"
    has_systemctl = True

    if run("which systemctl", verbose=False) != 0:
        has_systemctl = False
    if not has_systemctl:
        cmd = "service targetd status -l"

    _, data = run(cmd, return_output=True)
    for line in data.splitlines():
        if "ERROR:root:SSL file: '/etc/target/targetd_key.pem' does not exist" in line:
            return errors

    print("FAIL: Service has failed with different error than expected or service is running!")
    errors.append("FAIL: Service has failed with different error than expected or service is running!")

    return errors


if __name__ == "__main__":
    errs = targetd_ssl_func_test()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var, write_var


def create_access_group():
    print("INFO: Creating AG.")
    errors = []

    ag_name = read_var("LSM_AG_NAME") + environ["fmf_init"].upper()
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID" + environ["fmf_init"].upper())

    config = list(yield_lsm_config())[0]
    if "smis" in config["protocol"]:
        print("smis protocol does not support ag creation on EMC. skipping.")
        return errors
    lsm = LibStorageMgmt(disable_check=True, **config)

    _, data = atomic_run(
        "Creating AG '%s'." % ag_name,
        command=lsm.access_group_create,
        name=ag_name,
        init=init_id,
        sys=sys_id,
        return_output=True,
        errors=errors,
    )

    for line in data.splitlines():
        if ag_name in line:
            ag_id = line.split()[0].strip()
            atomic_run(
                "Writing var LSM_AG_ID" + environ["fmf_init"].upper(),
                command=write_var,
                var={"LSM_AG_ID" + environ["fmf_init"].upper(): ag_id},
                errors=errors,
            )

    return errors


if __name__ == "__main__":
    errs = create_access_group()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.fio import fio_stress
from libsan.host.vdo import VDO

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var
from stqe.host.vdo import is_block_device


def xfs_stress():
    errors = []
    vdo = VDO(disable_check=True)

    fs_dir = read_var("FS_DIR")
    vdo_name = environ["fmf_vdo_name"]

    ret = is_block_device("/dev/mapper/%s" % vdo_name)
    if ret is not True:
        errors.append(ret)
        return errors

    arguments = [
        {
            "message": "Checking vdo status to get info on data.",
            "command": vdo.status,
            "name": vdo_name,
        },
        {
            "message": "Starting fio stress on FS on VDO",
            "command": fio_stress,
            "of": "%s/test.img" % fs_dir,
            "verbose": True,
            # fio settings
            "ioengine": environ["fmf_ioengine"],
            "direct": environ["fmf_direct"],
            "group_reporting": environ["fmf_group_reporting"],
            "bs": environ["fmf_bs"],
            "iodepth": environ["fmf_iodepth"],
            "numjobs": environ["fmf_numjobs"],
            "rw": environ["fmf_rw"],
            "rwmixread": environ["fmf_rwmixread"],
            "randrepeat": environ["fmf_randrepeat"],
            "norandommap": environ["fmf_norandommap"],
            "size": environ["fmf_size"],
            "runtime": int(environ["fmf_runtime"]) * 60,
        },
        {
            "message": "Checking vdo status to get info on data.",
            "command": vdo.status,
            "name": vdo_name,
        },
    ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = xfs_stress()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, get_loopdev

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var, write_var


def setup_loop_devs(number_of_loopdevs):
    errors = []

    if not number_of_loopdevs:
        number_of_loopdevs = 2

    for i in range(number_of_loopdevs):
        atomic_run("Creating loop device number %s" % i, command=create_loopdev, errors=errors)

    loopdevs = atomic_run(message="Gettting loop devices", command=get_loopdev, errors=errors)

    blockdevs = read_var("STRATIS_DEVICE")

    if blockdevs:
        # backup the previous devices
        atomic_run(
            "Writing var STRATIS_DEVICE_BACKUP",
            command=write_var,
            var={"STRATIS_DEVICE_BACKUP": " ".join(blockdevs)},
            errors=errors,
        )
        if not isinstance(blockdevs, list):
            blockdevs = [blockdevs]
        loopdevs += [x for x in blockdevs if x not in blockdevs]

    if number_of_loopdevs and len(loopdevs) < number_of_loopdevs:
        msg = "WARN: Found only {} disks, need {} disks.".format(
            len(loopdevs),
            number_of_loopdevs,
        )
        print(msg)
        errors.append(msg)

    print("Using these blockdevs: %s" % " ".join(loopdevs))
    for disk in loopdevs:
        atomic_run(
            "Zeroing superblock of disk %s." % disk,
            command=run,
            cmd="dd if=/dev/zero of=%s bs=1M count=10" % disk,
            errors=errors,
        )
        atomic_run(
            "remove multipath superblock of disk %s." % disk,
            command=run,
            cmd="multipath -W %s" % disk,
            errors=errors,
        )

    atomic_run(
        "Writing var STRATIS_DEVICE",
        command=write_var,
        var={"STRATIS_DEVICE": loopdevs},
        errors=errors,
    )

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    try:
        number_of_loopdevs = read_env("fmf_number_of_disks")
    except KeyError:
        number_of_loopdevs = None
    errs = setup_loop_devs(number_of_loopdevs)
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def access_group_create_success():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Creating AG {} with protocol {}.".format(ag_name, config["protocol"]),
            command=lsm.access_group_create,
            name=ag_name,
            init=init_id,
            sys=sys_id,
            return_output=True,
            errors=errors,
        )
        ag_id = [line.split()[0].strip() for line in data.splitlines() if ag_name in line][0]
        atomic_run(
            "Deleting AG {} with protocol {}.".format(ag_name, config["protocol"]),
            command=lsm.access_group_delete,
            ag=ag_id,
            force=True,
            errors=errors,
        )
    return errors


def access_group_create_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail creating AG without any paramethers with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
            },
            {
                "message": "Trying to fail creating AG without name with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "sys": sys_id,
                "init": init_id,
            },
            {
                "message": "Trying to fail creating AG without init with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "sys": sys_id,
                "name": ag_name,
            },
            {
                "message": "Trying to fail creating AG without sys with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "init": init_id,
                "name": ag_name,
            },
            {
                "message": "Trying to fail creating AG with wrong init '-1' with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "init": -1,
                "name": ag_name,
                "sys": sys_id,
            },
            {
                "message": "Trying to fail creating AG with NONEXISTENT sys with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "init": init_id,
                "name": ag_name,
                "sys": "NONEXISTENT",
            },
            {
                "message": "Trying to fail creating AG with NONEXISTENT init with protocol %s" % config["protocol"],
                "command": lsm.access_group_create,
                "init": "NONEXISTENT",
                "name": ag_name,
                "sys": sys_id,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def access_group_create_fail_name_conflict():
    errors = []

    ag_name = read_var("LSM_AG_NAME")
    sys_id = read_var("LSM_SYS_ID")
    init_id = read_var("LSM_INIT_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Creating AG {} to hit NAME CONFLICT with protocol {}.".format(ag_name, config["protocol"]),
            command=lsm.access_group_create,
            name=ag_name,
            init=init_id,
            sys=sys_id,
            expected_ret=4,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = access_group_create_success()
    if int(environ["fmf_tier"]) == 2:
        errs = access_group_create_fail()
        errs += access_group_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python


import sys

from libsan.host import dm, linux, lvm, snapper
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev

import stqe.host.tc

#
# Bugs related:
#   BZ1365555 - Revert workaround for issues with snapper and btrfs subvolume labels
#

TestObj = None

loop_dev = None

vg_name = "vgtest"
pool_name = "test_pool"
thin1_name = "thin1"
brtfs_name = "brtfs_lv"
mnt_point_dev = "/mnt/snapper_test"
cfg_name = "bugtest"


def create_test_lv(filesystem):
    if not filesystem:
        return None

    # On brtfs we should not use thinp
    if filesystem != "btrfs":
        if run(f"lvcreate --thinpool {pool_name} -L 800M {vg_name}") != 0:
            print(f"FAIL: Could not create {pool_name} - {filesystem}")
            return None
        if run(f"lvcreate -V 1G -T {vg_name}/{pool_name} -n {thin1_name}") != 0:
            print(f"FAIL: Could not create {thin1_name} - {filesystem}")
            return None

        return thin1_name
    else:
        if run(f"lvcreate -L 500M  -n {brtfs_name} {vg_name}") != 0:
            print(f"FAIL: Could not create {brtfs_name} - {filesystem}")
            return None

        return brtfs_name


def start_test(filesystem, snapper_debug_mode):
    global TestObj

    print(80 * "#")
    print(f"INFO: Starting test on using snapper on {filesystem} debug mode:{snapper_debug_mode}")
    print(80 * "#")

    if snapper_debug_mode:
        # Wait 60 seconds so snapperd terminates
        linux.sleep(60)
        if not snapper.snapper_enable_debug_mode():
            TestObj.tfail("Could not enable snapper debug on FS(%s)" % filesystem)
            return False

    # create loop device of 900M
    global loop_dev
    loop_dev = create_loopdev(size=900)
    if not loop_dev:
        TestObj.tfail("Could not create loop device")
        return False

    if not lvm.vg_create(vg_name, loop_dev):
        TestObj.tfail('Could not create VG "%s"' % vg_name)
        return False

    print("INFO: Creating LVs")
    test_lv = create_test_lv(filesystem)
    if not test_lv:
        TestObj.tfail(f"Could not LV for FS({filesystem}) with snapper_debug_mode={snapper_debug_mode}")
        return False

    dm.dm_show_table()

    test_device = f"/dev/mapper/{vg_name}-{test_lv}"
    print(f"INFO:Going to create FS({filesystem}) on {test_device}")
    fs_option = ""
    if filesystem == "btrfs":
        # use mixed option as device is not big. more details see:
        # https://bugzilla.redhat.com/show_bug.cgi?id=1374158#c19
        fs_option = "--mixed"
    if run(f"mkfs.{filesystem} {fs_option} {test_device}") != 0:
        TestObj.tfail(
            f"Could not create FS({filesystem}) on {test_device} with snapper_debug_mode={snapper_debug_mode}",
        )
        return False

    if not linux.mkdir(mnt_point_dev):
        TestObj.tfail(
            "Could not create directory %s - FS(%s) with snapper_debug_mode=%s"
            % (mnt_point_dev, filesystem, snapper_debug_mode),
        )
        return False

    if not linux.mount(test_device, mnt_point_dev):
        TestObj.tfail(
            f"Could not mount {mnt_point_dev} - FS({filesystem}) with snapper_debug_mode={snapper_debug_mode}",
        )
        return False

    lvm.lv_show()

    print("INFO: Going create snapper config %s" % cfg_name)
    snap_type = "lvm(%s)" % filesystem
    if filesystem == "btrfs":
        snap_type = "btrfs"
    if not snapper.snapper_create_config(cfg_name, snap_type, mnt_point_dev):
        TestObj.tfail(
            f"Could not create snapper config - FS({filesystem}) with snapper_debug_mode={snapper_debug_mode}",
        )
        return False

    run("ps -ef | grep snapper")

    if not snapper.snapper_list_configs():
        TestObj.tfail(
            "Could not list config FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    # list current snapshots
    # do not expect any snaphost to be listed
    if not snapper.snapper_list(cfg_name):
        TestObj.tfail(
            "Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Creating single snapshot")
    pre_num = snapper.snapper_create(cfg_name, "single")
    if not pre_num:
        TestObj.tfail(
            "Could not create SINGLE snapshot FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not linux.mkdir("%s/dir_0" % mnt_point_dev):
        TestObj.tfail(
            "Could not create directory on %s - FS(%s) with snapper_debug_mode=%s"
            % (mnt_point_dev, filesystem, snapper_debug_mode),
        )
        return False
    if run("touch %s/dir_0/file_0" % mnt_point_dev) != 0:
        TestObj.tfail(
            "Could not ecreate file FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Creating pre snapshot")
    pre_num = snapper.snapper_create(cfg_name, "pre")
    if not pre_num:
        TestObj.tfail(
            "Could not create PRE snapshot FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail(
            "Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Creating data on %s" % mnt_point_dev)
    if not linux.mkdir("%s/dir_1" % mnt_point_dev):
        TestObj.tfail(
            "Could not create directory on %s - FS(%s) with snapper_debug_mode=%s"
            % (mnt_point_dev, filesystem, snapper_debug_mode),
        )
        return False
    if run("touch %s/dir_1/file_1" % mnt_point_dev) != 0:
        TestObj.tfail(
            "Could not create file FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False
    if run('echo "going to create snapshot" >> %s/dir_0/file_0' % mnt_point_dev) != 0:
        TestObj.tfail(
            "Could not edit file FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Creating post snapshot")
    post_num = snapper.snapper_create(cfg_name, "post", pre_num=pre_num)
    if not post_num:
        TestObj.tfail(
            "Could not create POST snapshot FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail(
            "Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not snapper.snapper_status(cfg_name, pre_num, post_num):
        TestObj.tfail(
            "Could not get status for snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not snapper.snapper_diff(cfg_name, pre_num, post_num):
        TestObj.tfail(
            "Could not get diff for snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Deleting post snapshot")
    if not snapper.snapper_delete(cfg_name, post_num):
        TestObj.tfail(
            "Could not delete POST snapshot FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    if not snapper.snapper_list(cfg_name):
        TestObj.tfail(
            "Could not list snapshots FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Deleting pre snapshot")
    if not snapper.snapper_delete(cfg_name, pre_num):
        TestObj.tfail(
            "Could not delete PRE snapshot FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    print("INFO: Deleting config")
    if not snapper.snapper_delete_config(cfg_name):
        TestObj.tfail(
            "Could not delete config FS(%s) on %s with snapper_debug_mode=%s"
            % (filesystem, test_device, snapper_debug_mode),
        )
        return False

    # Wait 60 seconds so snapperd terminates
    linux.sleep(60)
    if snapper_debug_mode and not snapper.snapper_disable_debug_mode():
        TestObj.tfail("Could not disable snapper debug on FS(%s)" % filesystem)
        return False

    print(80 * "#")
    TestObj.tpass(f"PASS: Test on FS {filesystem} and snapper_debug_mode={snapper_debug_mode}")
    print(80 * "#")

    return True


def _clean_up():
    global vg_name, loop_dev

    snapper.snapper_delete_config(cfg_name)

    linux.umount(mnt_point_dev)

    # make sure any failed device is removed
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail('Could not delete VG "%s"' % vg_name)

    if loop_dev:
        if not lvm.pv_remove(loop_dev):
            TestObj.tfail('Could not delete PV "%s"' % loop_dev)
        delete_loopdev(loop_dev)


def execute_test():
    global TestObj

    os_version = linux.dist_ver()
    os_name = linux.dist_name()

    if os_name == "RHEL" and os_version < 7:
        TestObj.tskip(f"Test is not supported on {os_name}-{os_version}")
        return

    linux.install_package("snapper")

    filesystems = ["ext4", "xfs"]
    # btrfs is depreciated in RHEL 7.4, but should remain accessible in RHEL 7 series
    # also btrfs was purposely removed from Pegas Kernel (RHEL ALT)
    ret, kernel = run(cmd="uname -r", return_output=True)
    if ret:
        print("WARN: Could not get kernele version, running without btrfs.")
    else:
        version = kernel.split("-")[0].split(".")
        # FIXME: add check for RHEL < 8
        if int(version[0]) <= 4 and int(version[1]) <= 11:
            filesystems.append("btrfs")
    # test using snapperd with and without debug
    dbg_modes = [False, True]

    for fs in filesystems:
        for dbg_mode in dbg_modes:
            _clean_up()

            if not start_test(fs, dbg_mode):
                log_name = f"snapper_{fs}_{dbg_mode}.log"
                run("mv /var/log/snapper.log %s" % log_name)
                TestObj.log_submit(log_name)

    _clean_up()
    return


def main():
    global TestObj

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("snapper")
    linux.install_package("lvm2")

    execute_test()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python


from time import sleep

from libsan.host import linux, targetd
from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_no_passwd_test():
    errors = []

    atomic_run("Disabling SSL", command=targetd.ssl_change, errors=errors)

    atomic_run("Removing password from config", command=targetd.set_password, errors=errors)

    atomic_run(
        "Restarting targetd service",
        service_name="targetd",
        command=linux.service_restart,
        errors=errors,
    )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run(
        "Checking retcode of targetd",
        retcode=3,
        command=targetd.targetd_status,
        errors=errors,
    )

    cmd = "systemctl status targetd"
    has_systemctl = True

    if run("which systemctl", verbose=False) != 0:
        has_systemctl = False
    if not has_systemctl:
        cmd = "service targetd status"

    _, data = run(cmd, return_output=True)

    for line in data.splitlines():
        if "CRITICAL:root:password" in line:
            return errors

    print("FAIL: Service has failed with different error than expected or service is running!")
    errors.append("FAIL: Service has failed with different error than expected or service is running!")
    return errors


if __name__ == "__main__":
    errs = targetd_no_passwd_test()
    exit(parse_ret(errs))



#!/usr/bin/python


from shutil import copy

from libsan.host.cmdline import run
from libsan.host.linux import compare_version
from libsan.host.vdo import VDOStats
from libsan.misc import time

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_env, read_var
from stqe.host.vdo import is_block_device


def single_copy(vdostats, device, vdo_name, fs_dir, file, duration):
    error = []
    file_name = "vdo_test.file"
    expected_ret = 0
    if compare_version("vdo", "6.2.0.0", "0") and not compare_version("vdo", "6.2.0.71", "71-4"):
        # Hit vdostats bug
        expected_ret = 1

    arguments = [
        {
            "message": f"Copying file {file} to {fs_dir}/{file_name}",
            "command": copy,
            "src": "%s" % file,
            "dst": f"{fs_dir}/{file_name}",
        },
        {
            "message": "Umounting filesystem",
            "command": run,
            "cmd": "umount %s" % fs_dir,
        },
        {
            "message": "Mounting XFS filesystem with discard enabled",
            "command": run,
            "cmd": f"mount -o discard {device} {fs_dir}",
        },
        {
            "message": f"Removing file {fs_dir}/{file_name}",
            "command": run,
            "cmd": f"rm -f {fs_dir}/{file_name}",
        },
        {
            "message": "Umounting filesystem",
            "command": run,
            "cmd": "umount %s" % fs_dir,
        },
        {
            "message": "Checking stats of vdo %s" % vdo_name,
            "command": vdostats.stats,
            "expected_ret": expected_ret,
        },
        {
            "message": "Mounting XFS filesystem with discard enabled",
            "command": run,
            "cmd": f"mount -o discard {device} {fs_dir}",
        },
    ]

    start_time = time.get_time(in_seconds=True)
    for argument in arguments:
        atomic_run(errors=error, **argument)
    end_time = time.get_time(in_seconds=True)
    total_time = time.time_2_sec(end_time - start_time)
    duration.append(total_time)

    return duration, error


def repeated_copy(repeat_count):
    errors = []
    vdostats = VDOStats()
    vdo_name = read_env("fmf_vdo_name")
    device = read_var("VDO_DEVICE")
    fs_dir = read_var("FS_DIR")
    file = read_var("DATA_FILE")

    ret = is_block_device(device)
    if ret is not True:
        errors.append(ret)
        return errors

    duration = []
    i = 0
    while i < repeat_count:
        i += 1
        print("############# Running iteration %s #############" % (i))
        duration, ret = single_copy(vdostats, device, vdo_name, fs_dir, file, duration)
        errors += ret
        print("######### This iteration took %s  #########" % time.sec_2_time(duration[-1]))

        # Reduce iteration count in case the iteration took too long.
        try:
            max_runtime = read_env("fmf_max_runtime")
            avg_duration = sum(duration) / len(duration)
            count = int(max_runtime * 60 / avg_duration)
            if count < repeat_count:
                print(f"INFO: Changing total amount of iterations from {repeat_count} to {count} to avoid timeout.")
                repeat_count = count
        except KeyError:
            pass

    print("\n####### SUMMARY of %s iterations ########" % (i))
    print("######### Average duration: %s ##########" % time.sec_2_time(int(sum(duration) / len(duration))))
    print("######### Minimum duration: %s ##########" % time.sec_2_time(min(duration)))
    print("######### Maximum duration: %s ##########\n" % time.sec_2_time(max(duration)))
    return errors


if __name__ == "__main__":
    if read_env("fmf_tier") == 1:
        errs = repeated_copy(read_env("fmf_repeat_count"))
    if read_env("fmf_tier") == 2:
        errs = repeated_copy(read_env("fmf_repeat_count"))
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_clone_success():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Cloning FS {} to FS {} with protocol {}.".format(fs_id, clone_name, config["protocol"]),
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=clone_name,
            return_output=True,
            errors=errors,
        )
        cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]
        atomic_run(
            "Removing cloned FS {} with protocol {}.".format(clone_name, config["protocol"]),
            command=lsm.fs_delete,
            fs=cloned_fs_id,
            force=True,
            errors=errors,
        )
        # FIXME: Add test with backing-snapshot
    return errors


def fs_clone_fail():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail cloning FS without any paramethers with protocol %s" % config["protocol"],
                "command": lsm.fs_clone,
            },
            {
                "message": "Trying to fail cloning FS without 'dst-name' with protocol %s" % config["protocol"],
                "command": lsm.fs_clone,
                "src_fs": fs_id,
            },
            {
                "message": "Trying to fail cloning FS without 'src-fs' with protocol %s" % config["protocol"],
                "command": lsm.fs_clone,
                "dst_name": clone_name,
            },
            {
                "message": "Trying to fail cloning FS with wrong 'src-fs' 'wrong' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_clone,
                "dst_name": clone_name,
                "src_fs": "wrong",
            },
            {
                "message": "Trying to fail cloning FS by giving 'src-fs' name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_clone,
                "dst_name": clone_name,
                "src_fs": fs_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def fs_clone_fail_name_conflict():
    errors = []

    clone_name = read_var("LSM_FS_CLONED_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Cloning FS {} to FS {} with protocol {}.".format(fs_id, clone_name, config["protocol"]),
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=clone_name,
            return_output=True,
            errors=errors,
        )
        cloned_fs_id = [line.split()[0].strip() for line in data.splitlines() if clone_name in line][0]

        atomic_run(
            "Cloning FS %s again to hit NAME CONFLICT with cloned FS name with protocol %s."
            % (fs_name, config["protocol"]),
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=clone_name,
            return_output=True,
            expected_ret=4,
            errors=errors,
        )

        atomic_run(
            "Cloning FS %s again to hit NAME CONFLICT with origin FS name with protocol %s."
            % (fs_name, config["protocol"]),
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=fs_name,
            return_output=True,
            expected_ret=4,
            errors=errors,
        )

        atomic_run(
            "Removing cloned FS {} with protocol {}.".format(clone_name, config["protocol"]),
            command=lsm.fs_delete,
            fs=cloned_fs_id,
            force=True,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_clone_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_clone_fail()
        errs += fs_clone_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import translate_cache_policy, yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_write_cache_policy_update_success():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Getting write cache policy of volume %s." % vol,
            command=lsm.volume_cache_info,
            vol=vol,
            return_output=True,
            script=True,
            errors=errors,
        )
        old_policy = None
        for line in data.splitlines():
            if "Write Cache Policy" in line:
                old_policy = translate_cache_policy(line.split().pop().strip())

        for policy in ["WB", "AUTO", "WT", "wb", "auto", "wt"]:
            atomic_run(
                "Updating policy of volume {} to be {} with protocol {}.".format(vol, policy, config["protocol"]),
                command=lsm.volume_write_cache_policy_update,
                vol=vol,
                policy=policy,
                errors=errors,
            )

        atomic_run(
            "Returning write policy of volume %s to former state '%s' with protocol %s."
            % (vol, old_policy, config["protocol"]),
            command=lsm.volume_write_cache_policy_update,
            vol=vol,
            policy=old_policy,
            errors=errors,
        )
    return errors


def volume_write_cache_policy_update_fail():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail changing write policy of volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_write_cache_policy_update,
            },
            {
                "message": "Trying to fail changing write policy of volume with WRONG volume with protocol %s"
                % config["protocol"],
                "vol": "WRONG",
                "policy": "auto",
                "command": lsm.volume_write_cache_policy_update,
            },
            {
                "message": "Trying to fail changing write policy of volume with WRONG policy with protocol %s"
                % config["protocol"],
                "vol": vol,
                "policy": "WRONG",
                "command": lsm.volume_write_cache_policy_update,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_write_cache_policy_update_offline():
    errors = []
    # TODO: Check updating policy of offline volume, create test
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_write_cache_policy_update_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_write_cache_policy_update_fail()
    exit(parse_ret(errs))
